package com.fjhx.common.service.logistics.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.entity.logistics.dto.LogisticsAmountDto;
import com.fjhx.common.entity.logistics.dto.LogisticsAmountSelectDto;
import com.fjhx.common.entity.logistics.po.LogisticsAmount;
import com.fjhx.common.entity.logistics.vo.LogisticsAmountVo;
import com.fjhx.common.mapper.logistics.LogisticsAmountMapper;
import com.fjhx.common.service.logistics.LogisticsAmountService;
import com.fjhx.common.utils.Assert;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 物流发生额 服务实现类
 * </p>
 *
 * @author
 * @since 2024-05-06
 */
@Service
public class LogisticsAmountServiceImpl extends ServiceImpl<LogisticsAmountMapper, LogisticsAmount> implements LogisticsAmountService {

    @Override
    public List<LogisticsAmountVo> getList(LogisticsAmountSelectDto dto) {
        IWrapper<LogisticsAmount> wrapper = getWrapper();

        wrapper.eq("la", LogisticsAmount::getYear, dto.getYear());
        wrapper.eq("la", LogisticsAmount::getLogisticsCompanyId, dto.getLogisticsCompanyId());

        wrapper.orderByDesc("la", LogisticsAmount::getYear);
        List<LogisticsAmountVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public void saveOrEdit(LogisticsAmountDto dto) {
        if (ObjectUtil.isEmpty(dto.getId())) {
            if (this.count(q -> q.eq(LogisticsAmount::getYear, dto.getYear())) > 0) {
                throw new ServiceException(StrUtil.format("已存在{}年的数据记录，请直接修改不要重复添加！", dto.getYear()));
            }
        }
        this.saveOrUpdate(dto);
    }

    @DSTransactional
    @Override
    public void saveOrEditList(LogisticsAmountDto dto) {
        Assert.notEmpty(dto.getLogisticsCompanyId(), "物流公司不能为空！");

        List<LogisticsAmount> logisticsAmountList = dto.getLogisticsAmountList();
        for (LogisticsAmount logisticsAmount : logisticsAmountList) {
            Assert.notEmpty(logisticsAmount.getYear(), "年份不能为空！");

            logisticsAmount.setLogisticsCompanyId(dto.getLogisticsCompanyId());
        }
        this.editLinked(logisticsAmountList, LogisticsAmount::getLogisticsCompanyId, dto.getLogisticsCompanyId());
    }

}
