package com.fjhx.form.controller;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.form.entity.*;
import com.fjhx.form.service.ReportService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.enums.BusinessType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

@RestController
@RequestMapping("/report")
public class ReportController {

    private final ReportService reportService;

    @Autowired
    public ReportController(ReportService reportService) {
        this.reportService = reportService;
    }

    /**
     * 生产端报表
     */
    @PostMapping("/productionReport")
    Page<ProductionReportBo> productionReport(@RequestBody ProductionReportSelectDto dto) {
        return reportService.productionReport(dto);
    }

    /**
     * 业务端报表
     */
    @PostMapping("/saleReport")
    Page<SaleReportBo> saleReport(@RequestBody ProductionReportSelectDto dto) {
        return reportService.saleReport(dto);
    }

    /**
     * 已完工未出货统计
     */
    @PostMapping("/waitShipmentReportStatistic")
    List<WaitShipmentReportStatisticBo> waitShipmentReportStatistic() {
        return reportService.waitShipmentReportStatistic();
    }

    /**
     * 已完工未出货报表
     */
    @PostMapping("/waitShipmentReport")
    Page<WaitShipmentReportBo> waitShipmentReport(@RequestBody ProductionReportSelectDto dto) {
        return reportService.waitShipmentReport(dto);
    }

    /**
     * 订单汇总
     */
    @PostMapping("/orderSummary")
    Page<OrderSummaryBo> orderSummary(@RequestBody OrderSummarySelectDto dto) {
        return reportService.orderSummary(dto);
    }

    /**
     * 订单汇总Excel导出
     */
    @Log(title = "订单汇总", businessType = BusinessType.EXPORT)
    @PostMapping("/orderSummaryExcelExport")
    void orderSummaryExcelExport(@RequestBody OrderSummarySelectDto dto, HttpServletResponse httpServletResponse) {
        reportService.orderSummaryExcelExport(dto, httpServletResponse);
    }

    /**
     * 应收余额核对
     */
    @PostMapping("/finance/receivableCheck")
    List<ReceivableCheckBo> receivableCheck(@RequestBody ReceivableCheckSelectDto dto) {
        return reportService.receivableCheck(dto);
    }

    /**
     * 帐龄分析
     */
    @PostMapping("/finance/agingAnalysis")
    List<ReceivableCheckBo> agingAnalysis(@RequestBody ReceivableCheckSelectDto dto) {
        return reportService.receivableCheck(dto);
    }

    /**
     * 帐龄分析
     */
    @PostMapping("/finance/receivableAccounts")
    List<JSONObject> receivableAccounts(@RequestBody ReceivableAccountsSelectDto dto) {
        return reportService.getReceivableAccounts(dto);
    }

    /**
     * 员工绩效
     */
    @PostMapping("/userPerformanceReport")
    List<UserPerformanceBo> userPerformanceReport(@RequestBody UserPerformanceSelectDto dto) {
        return reportService.userPerformanceReport(dto);
    }

}
