package com.fjhx.form.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.account.entity.account.dto.AccountRunningWaterDto;
import com.fjhx.account.entity.account.po.AccountRunningWater;
import com.fjhx.account.service.account.AccountRunningWaterService;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.utils.Assert;
import com.fjhx.common.utils.excel.util.ExcelUtil;
import com.fjhx.form.entity.*;
import com.fjhx.form.entity.calculate.po.CalculateItemBalance;
import com.fjhx.form.entity.payable.po.PayableBalance;
import com.fjhx.form.mapper.FinanceReportMapper;
import com.fjhx.form.service.FinanceReportService;
import com.fjhx.form.service.calculate.CalculateItemBalanceService;
import com.fjhx.form.service.payable.PayableBalanceService;
import com.fjhx.supply.service.supplier.SupplierInfoService;
import com.fjhx.wms.service.warehouse.WarehouseService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import org.apache.commons.codec.Charsets;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.net.URLEncoder;
import java.util.*;

@Service
public class FinanceReportServiceImpl implements FinanceReportService {
    @Resource
    private FinanceReportMapper financeReportMapper;
    @Resource
    private CalculateItemBalanceService calculateItemBalanceService;
    @Resource
    private AccountRunningWaterService accountRunningWaterService;
    @Resource
    private SupplierInfoService supplierInfoService;
    @Resource
    private WarehouseService warehouseService;

    @Resource
    private PayableBalanceService payableBalanceService;

    @Override
    public List<SubjectBalanceBo> subjectBalance(SubjectBalanceSelectDto dto) {
        Assert.notEmpty(dto.getYear(), "年份不能为空！");
        IWrapper<Object> wrapper = IWrapper.getWrapper();


        wrapper.apply("YEAR ( t1.expenses_time ) = {0}", dto.getYear());

        wrapper.keyword(dto.getKeyword(),
                new SqlField("asj.subjects_name"),
                new SqlField("t1.calculate_item_str")
        );


        Map<String, BigDecimal> subjectsBalanceMap = calculateItemBalanceService.mapKV(
                CalculateItemBalance::getIdStr,
                CalculateItemBalance::getBalanceAmount,
                q -> q.eq(CalculateItemBalance::getYear, dto.getYear() - 1)
        );


        SubjectBalanceBo totalBo = new SubjectBalanceBo();


        List<SubjectBalanceBo> subjectBalanceBos = financeReportMapper.subjectBalance(wrapper, dto);
        for (SubjectBalanceBo subjectBalanceBo : subjectBalanceBos) {
            BigDecimal month1Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth1Amount(), BigDecimal.ZERO);
            BigDecimal month2Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth2Amount(), BigDecimal.ZERO);
            BigDecimal month3Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth3Amount(), BigDecimal.ZERO);
            BigDecimal month4Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth4Amount(), BigDecimal.ZERO);
            BigDecimal month5Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth5Amount(), BigDecimal.ZERO);
            BigDecimal month6Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth6Amount(), BigDecimal.ZERO);
            BigDecimal month7Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth7Amount(), BigDecimal.ZERO);
            BigDecimal month8Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth8Amount(), BigDecimal.ZERO);
            BigDecimal month9Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth9Amount(), BigDecimal.ZERO);
            BigDecimal month10Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth10Amount(), BigDecimal.ZERO);
            BigDecimal month11Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth11Amount(), BigDecimal.ZERO);
            BigDecimal month12Amount = ObjectUtil.defaultIfNull(subjectBalanceBo.getMonth12Amount(), BigDecimal.ZERO);

            //获取期出余额
            BigDecimal yearBeginBalance = ObjectUtil.defaultIfNull(subjectsBalanceMap.get(subjectBalanceBo.getSubjectsId()), BigDecimal.ZERO);

            subjectBalanceBo.setYearBeginBalance(yearBeginBalance);
            //余额 = 年初余额 + (-年初借方) + 1-12月 + 本期发生
            //1-12月合计
            BigDecimal janToDecSum = month1Amount.add(month2Amount).add(month3Amount).add(month4Amount)
                    .add(month5Amount).add(month6Amount).add(month7Amount).add(month8Amount)
                    .add(month9Amount).add(month10Amount).add(month11Amount).add(month12Amount);
            //年初借方
            BigDecimal yearBeginBorrowAmount = ObjectUtil.defaultIfNull(subjectBalanceBo.getYearBeginBorrowAmount(), BigDecimal.ZERO);
            //计算余额
            BigDecimal balanceAmount = yearBeginBalance.subtract(yearBeginBorrowAmount).add(janToDecSum).add(subjectBalanceBo.getTradeAmount());
            subjectBalanceBo.setBalanceAmount(balanceAmount);

            totalBo.setSubjectsName("合计");
            totalBo.setYearBeginBalance(ObjectUtil.defaultIfNull(totalBo.getYearBeginBalance(), BigDecimal.ZERO).add(subjectBalanceBo.getYearBeginBalance()));
            totalBo.setYearBeginBorrowAmount(ObjectUtil.defaultIfNull(totalBo.getYearBeginBorrowAmount(), BigDecimal.ZERO).add(ObjectUtil.defaultIfNull(subjectBalanceBo.getYearBeginBorrowAmount(), BigDecimal.ZERO)));
            totalBo.setMonth1Amount(ObjectUtil.defaultIfNull(totalBo.getMonth1Amount(), BigDecimal.ZERO).add(month1Amount));
            totalBo.setMonth2Amount(ObjectUtil.defaultIfNull(totalBo.getMonth2Amount(), BigDecimal.ZERO).add(month2Amount));
            totalBo.setMonth3Amount(ObjectUtil.defaultIfNull(totalBo.getMonth3Amount(), BigDecimal.ZERO).add(month3Amount));
            totalBo.setMonth4Amount(ObjectUtil.defaultIfNull(totalBo.getMonth4Amount(), BigDecimal.ZERO).add(month4Amount));
            totalBo.setMonth5Amount(ObjectUtil.defaultIfNull(totalBo.getMonth5Amount(), BigDecimal.ZERO).add(month5Amount));
            totalBo.setMonth6Amount(ObjectUtil.defaultIfNull(totalBo.getMonth6Amount(), BigDecimal.ZERO).add(month6Amount));
            totalBo.setMonth7Amount(ObjectUtil.defaultIfNull(totalBo.getMonth7Amount(), BigDecimal.ZERO).add(month7Amount));
            totalBo.setMonth8Amount(ObjectUtil.defaultIfNull(totalBo.getMonth8Amount(), BigDecimal.ZERO).add(month8Amount));
            totalBo.setMonth9Amount(ObjectUtil.defaultIfNull(totalBo.getMonth9Amount(), BigDecimal.ZERO).add(month9Amount));
            totalBo.setMonth10Amount(ObjectUtil.defaultIfNull(totalBo.getMonth10Amount(), BigDecimal.ZERO).add(month10Amount));
            totalBo.setMonth11Amount(ObjectUtil.defaultIfNull(totalBo.getMonth11Amount(), BigDecimal.ZERO).add(month11Amount));
            totalBo.setMonth12Amount(ObjectUtil.defaultIfNull(totalBo.getMonth12Amount(), BigDecimal.ZERO).add(month12Amount));
            totalBo.setTradeAmount(ObjectUtil.defaultIfNull(totalBo.getTradeAmount(), BigDecimal.ZERO).add(subjectBalanceBo.getTradeAmount()));
            totalBo.setBalanceAmount(ObjectUtil.defaultIfNull(totalBo.getBalanceAmount(), BigDecimal.ZERO).add(subjectBalanceBo.getBalanceAmount()));
        }

        subjectBalanceBos.add(totalBo);

        return subjectBalanceBos;
    }

    /**
     * 每月1日0点1分 赋值客户上月科目余额
     */
//    @PostConstruct
    @Scheduled(cron = "0 1 0 1 * ?")
    public void autoSaveReceivableAccountsEnding() {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        SecurityUtils.setTenantId("000000");

        DateTime lastMonth = DateUtil.offsetMonth(new Date(), -1);

        SubjectBalanceSelectDto dto = new SubjectBalanceSelectDto();
        dto.setYear(DateUtil.year(lastMonth));
        List<SubjectBalanceBo> subjectBalanceBos = subjectBalance(dto);

        //每过一个月保存一次余额
        for (SubjectBalanceBo receivableAccount : subjectBalanceBos) {
            //获取上一月数据
            CalculateItemBalance lastCalculateItemBalance = calculateItemBalanceService.getOne(q -> q
                    .eq(CalculateItemBalance::getYear, dto.getYear())
                    .eq(CalculateItemBalance::getIdStr, receivableAccount.getSubjectsCalculateItemIds())
            );
            if (ObjectUtil.isEmpty(lastCalculateItemBalance)) {
                lastCalculateItemBalance = new CalculateItemBalance();
                lastCalculateItemBalance.setYear(dto.getYear());
                lastCalculateItemBalance.setIdStr(receivableAccount.getSubjectsCalculateItemIds());
            }

            //保存余额
            lastCalculateItemBalance.setBalanceAmount(receivableAccount.getBalanceAmount());
            calculateItemBalanceService.saveOrUpdate(lastCalculateItemBalance);
        }

        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.clear();
    }

    private void setRemitDetailPageWrapper(RemitDetailSelectDto dto, IWrapper wrapper) {
        wrapper.keyword(dto.getKeyword(),
                new SqlField("t1.contract_codes"),
                new SqlField("t2.subject_name"),
                new SqlField("t1.customerShortName"),
                new SqlField("t2.supplier_names"),
                new SqlField("cc.voucher_no"),
                new SqlField("arw.amount"),
                new SqlField("am.`alias`")
        );

        //交易时间范围过滤
        wrapper.ge("arw.transaction_time", dto.getBeginTime());
        wrapper.le("arw.transaction_time", dto.getEndTime());

        wrapper.orderByDesc("arw.transaction_time");
    }

    @Override
    public Page<RemitDetailVo> getRemitDetailPage(RemitDetailSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        //设置查询条件
        setRemitDetailPageWrapper(dto, wrapper);

        Page<RemitDetailVo> page = financeReportMapper.getRemitDetailPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public void remitDetailExcelExport(RemitDetailSelectDto dto, HttpServletResponse httpServletResponse) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        //设置查询条件
        setRemitDetailPageWrapper(dto, wrapper);

        List<RemitDetailVo> records = financeReportMapper.getRemitDetailPage(wrapper);
        //导出Excel
        ExcelUtil.export(httpServletResponse, records, RemitDetailVo.class);
    }

    @DSTransactional
    @Override
    public void editRemitDetail(AccountRunningWaterDto dto) {
        Assert.notEmpty(dto.getId(), "id不能为空！");
        accountRunningWaterService.update(q -> q
                .eq(AccountRunningWater::getId, dto.getId())
                .set(AccountRunningWater::getOppositeSubjectName, dto.getOppositeSubjectName())
                .set(AccountRunningWater::getSaleExpenseCategory, dto.getSaleExpenseCategory())
                .set(AccountRunningWater::getUpdateTime, new Date())
                .set(AccountRunningWater::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public Page<AccountsPayableVo> getAccountsPayablePage(AccountsPayableSelectDto dto) {
        Assert.notEmpty(dto.getYear(), "年份不能为空！");

        Page<AccountsPayableVo> accountsPayablePage = financeReportMapper.getAccountsPayablePage(dto.getPage(), IWrapper.getWrapper(), dto.getYear());
        List<AccountsPayableVo> records = accountsPayablePage.getRecords();

        //赋值详情
        setAccountsPayableInfo(records, dto);

        return accountsPayablePage;
    }

    /**
     * 应付账款详情
     */
    private void setAccountsPayableInfo(List<AccountsPayableVo> records, AccountsPayableSelectDto dto) {
        //获取年初数据
        Map<String, BigDecimal> lastEndBalanceMap = payableBalanceService.mapKV(PayableBalance::getIdStr, PayableBalance::getEndBalance, q -> q
                .eq(PayableBalance::getYear, dto.getYear() - 1));
        //获取当年数据
        Map<String, PayableBalance> balanceMap = payableBalanceService.mapKEntity(PayableBalance::getIdStr, q -> q.eq(PayableBalance::getYear, dto.getYear()));

        for (AccountsPayableVo record : records) {
            //赋值年初余额
            record.setYearBeginBalance(lastEndBalanceMap.getOrDefault(record.getIdStr(), BigDecimal.ZERO));

            //上月余额
            BigDecimal lastMonthEndBalance = record.getYearBeginBalance();

            //赋值备注
            PayableBalance payableBalance = balanceMap.getOrDefault(record.getIdStr(), new PayableBalance());
            record.setRemark(payableBalance.getRemark());

            //将当前年数据转JSON
            JSONObject balanceJson = JSONObject.parse(JSONObject.toJSONString(payableBalance));

            //处理月数据
            JSONObject dataJson = JSONObject.parse(record.getDataJson());
            JSONObject monthJson = new JSONObject();
            //赋值每月数据
            for (int i = 1; i <= 12; i++) {
                AccountsPayableVo.MonthDataVo monthData = dataJson.getObject(i + "", AccountsPayableVo.MonthDataVo.class);
                if (ObjectUtil.isNull(monthData)) {
                    monthData = new AccountsPayableVo.MonthDataVo();
                    monthData.setArrivalAmount(BigDecimal.ZERO);
                    monthData.setArrivalAdjustAmount(BigDecimal.ZERO);
                    monthData.setBackAmount(BigDecimal.ZERO);
                    monthData.setTaxAmount(BigDecimal.ZERO);
                    monthData.setTaxAdjustAmount(BigDecimal.ZERO);
                    monthData.setPaymentAmount(BigDecimal.ZERO);
                    monthData.setEndAmount(BigDecimal.ZERO);
                }

                //调整借方发生额 为负数
                monthData.setPaymentAmount(BigDecimal.ZERO.subtract(monthData.getPaymentAmount()));

                //赋值税费调整
                BigDecimal monthTaxAdjust = balanceJson.getBigDecimal("month" + i + "TaxAdjust");
                if (ObjectUtil.isEmpty(monthTaxAdjust)) {
                    monthTaxAdjust = BigDecimal.ZERO;
                }
                monthData.setTaxAdjustAmount(monthTaxAdjust);

                monthJson.put(i + "", monthData);

                //赋值余额
                BigDecimal add = lastMonthEndBalance.add(monthData.getArrivalAmount()).add(monthData.getBackAmount()).add(monthData.getArrivalAdjustAmount())
                        .add(monthData.getTaxAmount()).add(monthTaxAdjust).add(monthData.getPaymentAmount());
                monthData.setEndAmount(add);
                lastMonthEndBalance = add;
            }
            record.setMonthData(monthJson);
        }
        //赋值供应商名称
        supplierInfoService.attributeAssign(records, AccountsPayableVo::getSupplierId, (item, supplier) -> {
            item.setSupplierName(supplier.getName());
            item.setSupplierShortName(supplier.getShortName());
            item.setSupplierTaxPoints(supplier.getPrivTaxPoints());
        });
        //赋值仓库名称
        warehouseService.attributeAssign(records, AccountsPayableVo::getWarehouseId, (item, warehouse) -> {
            item.setWarehouseName(warehouse.getName());
        });
    }

    @Override
    public void accountsPayableExportExcel(HttpServletResponse response, AccountsPayableSelectDto dto) {
        List<AccountsPayableVo> list = financeReportMapper.getAccountsPayablePage(IWrapper.getWrapper(), dto.getYear());

        //赋值详情
        setAccountsPayableInfo(list, dto);

        //数据处理
        List<List<Object>> writeList = new ArrayList<List<Object>>();
        for (AccountsPayableVo accountsPayableVo : list) {
            List<Object> item = new ArrayList<>();
            item.add(accountsPayableVo.getSupplierShortName());
            item.add(accountsPayableVo.getWarehouseName());
            item.add(accountsPayableVo.getSupplierTaxPoints());
            item.add(accountsPayableVo.getYearBeginBalance());

            JSONObject monthData = accountsPayableVo.getMonthData();
            for (int i = 1; i <= 12; i++) {
                AccountsPayableVo.MonthDataVo month = monthData.getObject(1 + "", AccountsPayableVo.MonthDataVo.class);
                item.add(month.getArrivalAmount());
                item.add(month.getBackAmount());
                item.add(month.getArrivalAdjustAmount());
                item.add(month.getTaxAmount());
                item.add(month.getTaxAdjustAmount());
                item.add(month.getPaymentAmount());
                item.add(month.getEndAmount());
            }

            item.add(accountsPayableVo.getRemark());

            writeList.add(item);
        }

        //标题
        List<List<String>> headList = new ArrayList<List<String>>();
        headList.add(Arrays.asList("", "简称"));
        headList.add(Arrays.asList("", "入库仓库"));
        headList.add(Arrays.asList("", "税点"));
        headList.add(Arrays.asList("", "年初余额"));
        for (int i = 1; i <= 12; i++) {
            List<String> asList = Arrays.asList("到货金额", "退货金额", "货款调整", "税费", "税费调整", "借方发生额", "期末余额");
            for (String ti : asList) {
                headList.add(Arrays.asList(i + "月", ti));
            }
        }
        headList.add(Arrays.asList("", "备注"));

        //导出
        try {
            response.setContentType("application/vnd.ms-excel");
            response.setCharacterEncoding(Charsets.UTF_8.name());
            String fileName = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_14) + "导出数据";
            fileName = URLEncoder.encode(fileName, Charsets.UTF_8.name());
            response.setHeader("Content-disposition", "attachment;filename=" + fileName + ".xlsx");
            EasyExcel.write(response.getOutputStream()).sheet().head(headList).doWrite(writeList);
        } catch (Exception e) {
            throw new ServiceException(e.getMessage());
        }
    }

    /**
     * 每月1日0点1分 赋值上月应付余额
     */
//    @PostConstruct
    @Scheduled(cron = "0 1 0 1 * ?")
    public void autoSaveAccountsPayablePageEnding() {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        SecurityUtils.setTenantId("000000");

        DateTime lastMonth = DateUtil.offsetMonth(new Date(), -1);
        int year = DateUtil.year(lastMonth);
        int month = DateUtil.month(lastMonth) + 1;

        AccountsPayableSelectDto dto = new AccountsPayableSelectDto();
        dto.setYear(year);
        List<AccountsPayableVo> list = financeReportMapper.getAccountsPayablePage(IWrapper.getWrapper(), dto.getYear());
        setAccountsPayableInfo(list, dto);

        for (AccountsPayableVo accountsPayableVo : list) {
            AccountsPayableVo.MonthDataVo monthData = accountsPayableVo.getMonthData().getObject(month + "", AccountsPayableVo.MonthDataVo.class);

            PayableBalance payableBalance = payableBalanceService.getOne(q -> q
                    .eq(PayableBalance::getIdStr, accountsPayableVo.getIdStr()).eq(PayableBalance::getYear, year));
            if (ObjectUtil.isEmpty(payableBalance)) {
                payableBalance = new PayableBalance();
                payableBalance.setIdStr(accountsPayableVo.getIdStr());
                payableBalance.setYear(year);
            }
            payableBalance.setEndBalance(monthData.getEndAmount());

            payableBalanceService.saveOrUpdate(payableBalance);
        }

        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.clear();
    }
}
