package com.fjhx.mes.service.production.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.production.dto.ProductionReportingDto;
import com.fjhx.mes.entity.production.dto.ProductionReportingSelectDto;
import com.fjhx.mes.entity.production.po.*;
import com.fjhx.mes.entity.production.vo.ProductionReportingDetailVo;
import com.fjhx.mes.entity.production.vo.ProductionReportingVo;
import com.fjhx.mes.entity.technology.po.TechnologyProcessLine;
import com.fjhx.mes.mapper.production.ProductionReportingMapper;
import com.fjhx.mes.service.production.*;
import com.fjhx.mes.service.technology.TechnologyProcessLineService;
import com.fjhx.tenant.utils.DeptUstil;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <p>
 * 生产报工 服务实现类
 * </p>
 *
 * @author
 * @since 2023-03-30
 */
@Service
public class ProductionReportingServiceImpl extends ServiceImpl<ProductionReportingMapper, ProductionReporting> implements ProductionReportingService {
    @Autowired
    ProductionReportingDetailService productionReportingDetailService;
    @Autowired
    ProductInfoService productInfoService;
    @Autowired
    ProductionProcessesService productionProcessesService;
    @Autowired
    private ProductionTaskProgressService productionTaskProgressService;
    @Autowired
    private ProduceOrderDetailService produceOrderDetailService;
    @Autowired
    private TechnologyProcessLineService technologyProcessLineService;
    @Autowired
    private ProduceOrderService produceOrderService;
    @Autowired
    private ProductionSchedulingService productionSchedulingService;

    @Override
    public Page<ProductionReportingVo> getPage(ProductionReportingSelectDto dto) {
        IWrapper<ProductionReporting> wrapper = getWrapper();

        //权限过滤：生产报工
        wrapper.in("pr", ProductionReporting::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("pr", ProductionReporting::getCompanyId, dto.getCompanyId());

        //工序过滤
        wrapper.eq("pr", ProductionReporting::getProductionProcessesId, dto.getProductionProcessesId());

        //用户过滤
        if (ObjectUtil.isNotEmpty(dto.getUserId())) {
            wrapper.apply("FIND_IN_SET( {0}, pr.user_set )", dto.getUserId());
        }

        //日期过滤
        if (ObjectUtil.isNotEmpty(dto.getDataDate())) {
            wrapper.apply("DATE_FORMAT(pr.create_time, '%Y-%m') = DATE_FORMAT({0}, '%Y-%m')", dto.getDataDate());
        }

        //关键字检索
        wrapper.keyword(dto.getKeyword(),
                new SqlField("pr.user_name"),
                new SqlField("po.code")
        );

        wrapper.orderByDesc("pr", ProductionReporting::getId);
        Page<ProductionReportingVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionReportingVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }

        //获取生产公司信息
        List<Long> companyIds = records.stream().map(ProductionReporting::getCompanyId).collect(Collectors.toList());
        Map<Long, String> companyNameMap = DeptUstil.getDeptNameMap(companyIds);

        for (ProductionReportingVo record : records) {
            //赋值生产公司名称
            record.setCompanyName(companyNameMap.get(record.getCompanyId()));
        }

        //赋值产品信息
        productInfoService.attributeAssign(records, ProductionReportingVo::getProductId, (item, productInfo) -> {
            item.setProductName(productInfo.getName());
            item.setProductCode(productInfo.getCustomCode());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());
        });

        return page;
    }

    @Override
    public ProductionReportingVo detail(Long id) {
        ProductionReportingVo productionReporting = baseMapper.detail(id);
        ProductInfo productInfo = productInfoService.getById(productionReporting.getProductId());
        if (ObjectUtil.isNotEmpty(productInfo)) {
            productionReporting.setProductName(productInfo.getName());
        }
        Map<Long, String> nickNameMapByIds = UserUtil.getNickNameMapByIds(Arrays.asList(productionReporting.getPersonLiableId()));
        if (ObjectUtil.isNotEmpty(nickNameMapByIds)) {
            String name = nickNameMapByIds.get(productionReporting.getPersonLiableId());
            productionReporting.setPersonLiableName(name);
        }

        //赋值报工明细
        List<ProductionReportingDetail> productionReportingDetailList = productionReportingDetailService.list(q -> q.eq(ProductionReportingDetail::getProductionReportingId, productionReporting.getId()));
        List<ProductionReportingDetailVo> productionReportingDetailVos = BeanUtil.copyToList(productionReportingDetailList, ProductionReportingDetailVo.class);
        //赋值工序名称
        List<Long> ProductionProcessesIds = productionReportingDetailList.stream().map(ProductionReportingDetail::getProductionProcessesId).collect(Collectors.toList());
        if (ObjectUtil.isNotEmpty(ProductionProcessesIds)) {
            List<ProductionProcesses> productionProcessesList = productionProcessesService.listByIds(ProductionProcessesIds);
            Map<Long, ProductionProcesses> productionProcessesMap = productionProcessesList.stream().collect(Collectors.groupingBy(ProductionProcesses::getId,
                    Collectors.collectingAndThen(Collectors.toList(), value -> value.get(0))));
            for (ProductionReportingDetailVo productionReportingDetailVo : productionReportingDetailVos) {
                Long productionProcessesId = productionReportingDetailVo.getProductionProcessesId();
                ProductionProcesses productionProcesses = productionProcessesMap.get(productionProcessesId);
                if (ObjectUtil.isNotEmpty(productionProcesses)) {
                    productionReportingDetailVo.setProcessesName(productionProcesses.getName());
                }
            }
            productionReporting.setProductionReportingDetailVoList(productionReportingDetailVos);
        }
        return productionReporting;
    }

    @DSTransactional
    @Override
    public synchronized void add(ProductionReportingDto dto) {
        Long productionTaskId = dto.getProductionTaskId();
        Long productionProcessesId = dto.getProductionProcessesId();
        BigDecimal reportingQuantity = dto.getQuantity();
        String userSet = dto.getUserSet();

        Assert.notEmpty(userSet, "报工人不能为空");
        Assert.notEmpty(productionTaskId, "生产任务id不能为空！");
        Assert.notEmpty(productionProcessesId, "生产工序id不能为空！");
        Assert.notEmpty(reportingQuantity, "报工数量不能为空！");

        //赋值归属公司
        Long companyId = SecurityUtils.getCompanyId();
        dto.setCompanyId(companyId);

        //获取工艺信息
        ProductionOrderDetail productionOrderDetail = produceOrderDetailService.getById(productionTaskId);
        Assert.notEmpty(productionOrderDetail, "查询不到生产任务信息！");
        ProductInfo productInfo = productInfoService.getById(productionOrderDetail.getProductId());
        Assert.notEmpty(productInfo, "查询不到生产产品信息！");
        Long technologyId = productInfo.getTechnologyId();

        //获取第一道工序信息
        TechnologyProcessLine oneProcessInfo = technologyProcessLineService.getOne(q -> q
                .eq(TechnologyProcessLine::getTechnologyId, technologyId)
                .eq(TechnologyProcessLine::getSourceProcessesId, 1)
        );
        Assert.notEmpty(oneProcessInfo, "获取第一道工序信息失败！");

        //检查第一道工序是否排程
        if (productionSchedulingService.count(q -> q.eq(ProductionScheduling::getTaskId, productionTaskId).eq(ProductionScheduling::getProcessesId, oneProcessInfo.getTargetProcessesId())) <= 0) {
            throw new ServiceException("第一道工序未排程，禁止报工！");
        }

        //获取工序信息
        ProductionProcesses productionProcesses = productionProcessesService.getById(productionProcessesId);
        Assert.notEmpty(productionProcesses, "查询不到工序信息！");

        //赋值价格
        BigDecimal costPrice = ObjectUtil.defaultIfNull(productionProcesses.getCostPrice(), BigDecimal.ZERO);
        dto.setPrice(costPrice);
        dto.setAmount(reportingQuantity.multiply(dto.getPrice()));

        //保存报工
        this.save(dto);

        //生成报工明细
        long[] userIdList = Arrays.asList(userSet.split(",")).stream().mapToLong(Long::parseLong).toArray();
        //报工数量所有人平分
        BigDecimal divide = reportingQuantity.divide(BigDecimal.valueOf(userIdList.length), 4, RoundingMode.HALF_UP);
        List<ProductionReportingDetail> productionReportingDetailList = new ArrayList<>();
        for (long userId : userIdList) {
            ProductionReportingDetail productionReportingDetail = new ProductionReportingDetail();
            productionReportingDetail.setProductionReportingId(dto.getId());
            productionReportingDetail.setProductionTaskId(productionTaskId);
            productionReportingDetail.setProductionProcessesId(productionProcessesId);
            productionReportingDetail.setQuantity(divide);
            productionReportingDetail.setUserId(userId);
            productionReportingDetail.setCompanyId(companyId);
            //单价信息
            productionReportingDetail.setPrice(dto.getPrice());
            productionReportingDetail.setAmount(divide.multiply(dto.getPrice()));

            productionReportingDetailList.add(productionReportingDetail);
        }
        productionReportingDetailService.saveBatch(productionReportingDetailList);

        ProductionTaskProgress productionTaskProgress = productionTaskProgressService.getOne(q -> q
                .eq(ProductionTaskProgress::getTaskId, productionTaskId)
                .eq(ProductionTaskProgress::getProcessesId, productionProcessesId)
        );

        Assert.notEmpty(productionTaskProgress, "查询不到生产进度信息");

        Date newDateTime = new Date();

        //更新生产进度信息
        productionTaskProgressService.update(q -> q
                .eq(ProductionTaskProgress::getTaskId, productionTaskId)
                .eq(ProductionTaskProgress::getProcessesId, productionProcessesId)
                .setSql("finish_quantity = finish_quantity + " + dto.getQuantity())
                .setSql("balance_quantity = balance_quantity - " + dto.getQuantity())
                .set(BasePo::getUpdateTime, newDateTime)
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //更新完工数量
        //更新下一工序结存数量
        List<Long> nextProcessIds = technologyProcessLineService.listObject(TechnologyProcessLine::getTargetProcessesId, q -> q
                .eq(TechnologyProcessLine::getTechnologyId, technologyId)
                .eq(TechnologyProcessLine::getSourceProcessesId, productionProcessesId)
                .ne(TechnologyProcessLine::getTargetProcessesId, 99)
        );
        if (ObjectUtil.isNotEmpty(nextProcessIds)) {
            productionTaskProgressService.update(q -> q
                    .eq(ProductionTaskProgress::getTaskId, productionTaskId)
                    .in(ProductionTaskProgress::getProcessesId, nextProcessIds)
                    .setSql("balance_quantity = balance_quantity + " + dto.getQuantity())
                    .set(BasePo::getUpdateTime, newDateTime)
                    .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
            );
        }


        //判断报工工序是不是最后一道工序,是增加完工数量
        long count = technologyProcessLineService.count(q -> q
                .eq(TechnologyProcessLine::getTechnologyId, technologyId)
                .eq(TechnologyProcessLine::getSourceProcessesId, productionProcessesId)
                .eq(TechnologyProcessLine::getTargetProcessesId, 99)
        );
        if (count > 0) {
            //是最后一道工序添加完工数量，生成完工入库
            BigDecimal quantity = productionOrderDetail.getQuantity();
            BigDecimal finishQuantity = productionOrderDetail.getFinishQuantity();
            //计算完工数量
            BigDecimal add = finishQuantity.add(reportingQuantity);
            productionOrderDetail.setFinishQuantity(add);
            //全部完成赋值完工时间
            if (add.compareTo(quantity) >= 0) {
                productionOrderDetail.setFinishTime(newDateTime);
                productionOrderDetail.setProduceStatus(2);
            }
            produceOrderDetailService.updateById(productionOrderDetail);
        } else {
            produceOrderDetailService.update(q -> q
                    .eq(ProductionOrderDetail::getId, productionTaskId)
                    .eq(ProductionOrderDetail::getProduceStatus, 0)
                    .set(ProductionOrderDetail::getProduceStatus, 1)
            );
        }
        //检查订单下的任务是否完成，更改状态
        Long produceOrderId = productionOrderDetail.getProduceOrderId();
        long count1 = produceOrderDetailService.count(q -> q
                .eq(ProductionOrderDetail::getProduceOrderId, produceOrderId)
                .ne(ProductionOrderDetail::getProduceStatus, 2)
        );
        if (count1 == 0) {
            produceOrderService.update(q -> q
                    .eq(BasePo::getId, produceOrderId)
                    .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
                    .set(BasePo::getUpdateTime, newDateTime)
                    .set(ProductionOrder::getProduceStatus, 2)
                    .set(ProductionOrder::getFinishTime, newDateTime)
            );
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(ProductionReportingDto productionReportingDto) {
//        this.updateById(productionReportingDto);
//        //保存报工明细
//        List<ProductionReportingDetail> productionReportingDetailList = productionReportingDto.getProductionReportingDetailList();
//        for (ProductionReportingDetail productionReportingDetail : productionReportingDetailList) {
//            productionReportingDetail.setProductionReportingId(productionReportingDto.getId());
//        }
//        productionReportingDetailService.updateBatchById(productionReportingDetailList);
    }

    @DSTransactional
    @Override
    public void delete(Long id) {
        ProductionReporting dataById = this.getById(id);
        Assert.notEmpty(dataById, "查询不到报工信息！");
        Long taskId = dataById.getProductionTaskId();
        Long processesId = dataById.getProductionProcessesId();
        BigDecimal quantity = dataById.getQuantity();

        this.removeById(id);
        productionReportingDetailService.remove(q -> q.eq(ProductionReportingDetail::getProductionReportingId, id));

        //获取工艺信息
        ProductionOrderDetail pod = produceOrderDetailService.getById(taskId);
        Assert.notEmpty(pod, "查询不到生产任务信息！");
        ProductInfo productInfo = productInfoService.getById(pod.getProductId());
        Assert.notEmpty(productInfo, "查询不到生产产品信息！");
        Long technologyId = productInfo.getTechnologyId();

        //回滚当前工序生产进度
        productionTaskProgressService.update(q -> q
                .eq(ProductionTaskProgress::getTaskId, taskId)
                .eq(ProductionTaskProgress::getProcessesId, processesId)
                .setSql("finish_quantity = finish_quantity - " + quantity)
                .setSql("balance_quantity = balance_quantity + " + quantity)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //回滚下一工序结存数量
        List<Long> nextProcessIds = technologyProcessLineService.listObject(TechnologyProcessLine::getTargetProcessesId, q -> q
                .eq(TechnologyProcessLine::getTechnologyId, technologyId)
                .eq(TechnologyProcessLine::getSourceProcessesId, processesId)
                .ne(TechnologyProcessLine::getTargetProcessesId, 99)
        );
        if (ObjectUtil.isNotEmpty(nextProcessIds)) {
            productionTaskProgressService.update(q -> q
                    .eq(ProductionTaskProgress::getTaskId, taskId)
                    .in(ProductionTaskProgress::getProcessesId, nextProcessIds)
                    .setSql("balance_quantity = balance_quantity - " + quantity)
                    .set(BasePo::getUpdateTime, new Date())
                    .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
            );
        }

        //回滚生产任务完成数量以及状态
        produceOrderDetailService.update(q -> q
                .eq(ProductionOrderDetail::getId, taskId)
                .setSql("produce_status = IF(produce_status=2,1,produce_status)")
                .setSql("finish_quantity = finish_quantity - " + quantity)
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
        //回滚生产订单状态
        produceOrderService.update(q -> q
                .eq(ProductionOrder::getId, pod.getProduceOrderId())
                .setSql("produce_status = IF(produce_status=2,1,produce_status)")
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

}
